import $ from './util/dom-core.js';
import { gennerOneLine } from './util/lineUtil.js';
import Jquery from 'jquery';
import yzPage from './yzPage.js';
import { UA, isFunction, replaceHtmlSymbol, getParentByClassname, trim, throttle, isEmptyElement, getParentNodeByClass } from './util/util.js';

let yzPageManager = function (editor) {
  this.editor = editor;
  this.id = ('lsiten_editor_page_'+Math.random()).replace('.','_');
  this.pageIndex = 0;
  this.currentPage = null;
  this.pages = [];
  // enter键状态
  this._event = null;
  this._checkPageOutStatus = false;  
  this._checkPageBackStatus = false;  

  // 输入法监听 
  this.inputType = false;

  this.$el = $('<div id = " ' + this.id + ' " class="lsiten-pages-view"></div>');

  this.ready = false;
  this.editorPosition = {};
  this._$handleImg = null;
  this._init();
}

yzPageManager.prototype = {
  constructor: yzPageManager,
  _init: function () {
    this._initStyles();
    if (!this.editor.cardHtml ||  this.editor.cardHtml.length < 1) {
      this._addPage();
    }
    this._initEvents();
  },
  /**
   * function 初始化编辑后的数据
   */
  parseCardHtml: function (cardHtml) {
    const editor = this.editor;
    this._initStatus = true;
    this._removePages();
    cardHtml.forEach(pageHtml => {
      let $pageHtml = $(pageHtml);
      this._addPage($pageHtml);
    });
    this.pages.forEach(page => {
      page.$colums.forEach($colum => {
        page._updateCrossColumn($colum);
      })
    })
  },
  /**
   * function 删除所有页面
   */
  _removePages: function () {
    this.currentPage = null;
    this.pages = [];
    this.$el.html('');
    this.pageIndex = 0;
  },
  /**
   * function 初始化页面的样式
   */
  _initStyles: function () {
    this.$el.css('width', this._pagesWidth[this.editor.pageSize])
            .css('background', '#ffffff')
            .css('position', 'relative')
            .css('padding', '70px!important')
            .css('box-sizing', 'border-box')
            .css('min-width', '0')
            .css('box-shadow', '0 2px 7px rgba(0,0,0,.1)')
            .css('overflow', 'hidden')
            .css('margin', '0 auto')
            .css('height', this._pagesHeight[this.editor.pageSize]);
  },
  /**
   * function 添加一页
   */
  _addPage: function ($page) {
    this._addBreakPlaceholder();    
    let page = new yzPage(this,  $page || null);
    this.pages.push(page);
    this.pageIndex++;
    if (!$page) {
      page.addHeader();
    }
    return page;
  },
  /**
   * function 通过key值获取下一页
   * @param {number} key page的key值
   * @param {Boolean} isAddPage 如果没有是否添加一页
   */
  getNextPageByKey: function (key, isAddPage) {
    let page = this.pages[key];
    if (page) {
      return page;
    } else {
      if (isAddPage) {
        return this._addPage();
      } else {
        return false;
      }
    }
  },
  _addBreakPlaceholder: function () {
    if (this.pageIndex < 1) {
      return '';
    }
    // 添加分页显示效果
    let $breakDiv = $('<div class="page-break js-page-break-' + (this.pageIndex) + '" data-page-id="' + (this.pageIndex) + '"></div>');
    let $topSpan = $('<div class="spanMark topSpan">&#8203;</div>');
    $topSpan.css('display', 'block').css('height', '71.5px');
    // 分页符
    let $middleSpan = $('<div class="spanMark middleSpan lsiten-page-break-span">&#8203;</div>');
    $middleSpan.css('page-break-before', 'always');
    $middleSpan.css('display', 'block').css('height', '10px').css('background-color', '#f6f6f6').css('box-shadow', 'inset 0 7px 7px -5px rgba(0, 0, 0, 0.1), inset 0 -5px 5px -7px rgba(0, 0, 0, 0.1), 7px 0 7px #f6f6f6, -7px 0 7px #f6f6f6');  
    let $bottomSpan = $('<div class="spanMark bottomSpan">&#8203;</div>');
    $bottomSpan.css('height', '71.5px').css('display', 'block');

    // 添加页眉页脚   
    $breakDiv.append($topSpan);
    $breakDiv.append($middleSpan);
    $breakDiv.append($bottomSpan);
    $breakDiv.css('height', '153px');
    this.$el.append($breakDiv);

    let height = this.pages.length * this._pagesBreakHeight[this.editor.pageSize] + parseInt(this._pagesHeight[this.editor.pageSize]);
    this.$el.css('height', height + 'px');
  },

  _checkEveryPageOut: function () {
    if (this._checkPageOutStatus) {
      return false;
    }
    this._checkPageOutStatus = true;
    let pages = this.pages;
    pages.forEach(page => {
      page._checkContentOut();
    });
    this._checkPageOutStatus = false;
  },

  /**
   * function 检测是否有空页并删除
   */
  _checkEveryPageBack: function () {
    if (this._checkPageBackStatus) {
      return false;
    }
    this._checkPageBackStatus = true;
    let pages = this.pages;
    pages.forEach(page => {
      page._checkContentBack();
    });
    this._checkPageBackStatus = false;
  },

  _checkRemovePage() {
    let pagesTemp = this.pages.slice(0);
    let i = pagesTemp.length - 1;
    while(i>=0) {
      let page = pagesTemp[i];
      let currentParagraphs = page.$textElem.find('.js-paragraph-view');
      if (currentParagraphs.length && currentParagraphs.length === 1 && currentParagraphs[0].firstChild.className && currentParagraphs[0].firstChild.className.indexOf('js-answer-header') > -1) {
        this.removePage(page.pageIndex);
      }
      i--;
    }
  },
  /**
   * function 移除某一页
   * @param {int} 页面id
   */
  removePage: function (index) {
    this.$el.find('.js-page-break-' + index).remove();
    this.pages[index].$textElem.remove();
    this.pageIndex--;
    this.pages.splice(index,1);
    let i = 0;
    this.pages.forEach(page => {
      page.$textElem.find('.js-column').attr('data-column-page', i++);
    })
    let height = (this.pages.length - 1) * this._pagesBreakHeight[this.editor.pageSize] + parseInt(this._pagesHeight[this.editor.pageSize]);
    this.$el.css('height', height + 'px');
  },
  /**
   * function 添加事件监听 
   */
  _initEvents: function () {
    this._realtimeSavePosition();
    this._enterEvent();
    this._backEvent();
    this._delEvent();

    this._contentchangeEvent();

    // 框内事件菜单
    this._borderEvent();
  },
  /**
   * function 框相关事件
   */
  _borderEvent: function () {
    let _this = this;
    const $textElem = this.$el;
    Jquery($textElem[0]).on('contextmenu','.js-lsiten-border', (e) => {
      this._createContextMenu(e);
    })
  },
  /**
   * fucntion 创建菜单
   * @param {event} e 事件对象
   * @param {array} menus 数组对象
   */
  _createContextMenu: function (e, menus) {
    let $target = $(e.target);
    let $border = getParentByClassname($target, 'js-lsiten-border');
    let movex=e.pageX;
    let movey=e.pageY;
    !menus && (menus = []);
    this._createMenu(movex, movey, $border, menus);
    e.preventDefault();
  },
  /**
   * function 创建菜单列表
   * @param {number} x x轴坐标 
   * @param {number} y y轴坐标 
   * @param {DomElement} $border 该框 
   * @param {array} menus 数组对象 
   */
  _createMenu: function (x, y, $border, menus) {
    let $menu = $('<div class="js-lsiten-contentmenus"></div>');
    $menu.css('width', '100px')
         .css('height', 'auto')
         .css('background', '#fff')
         .css('position', 'absolute')
         .css('z-index', '9999')
         .css('outline', 'none')
         .css('top', y + 'px')
         .css('border', '1px solid #e4dddd')
         .css('padding', '0')
         .css('box-shadow', '0 3px 7px rgba(0, 0, 0, 0.1)')
         .css('left', x + 'px');
    $menu.attr('tabIndex', 1);

    menus.forEach(item => {
      let $menuItem = $('<div>' + item.name + '</div>');
      $menuItem.css('cursor', 'pointer')
           .css('text-align', 'center')
           .css('border-bottom', '1px solid #e8e3e3')
           .css('padding', '5px 0');
      $menu.append($menuItem);
      if (item.click && typeof item.click === 'function') {
        item.click($menuItem);
      }
    })
    this._commonMenus($menu, $border);
    $(document.body).append($menu);
    $menu.focus();
    Jquery($menu[0]).on('blur', (e) => {
      $menu && $menu.remove();
    })
  },
  /**
   * function 创建菜单列表
   * @param {DomElement} $menu menu的dom
   * @param {DomElement} $border 框的dom
   */
  _commonMenus: function ($menu, $border) {
    // 添加文本框开始
    let $addText = $('<div>加文本框</div>');
    $addText.css('cursor', 'pointer')
           .css('text-align', 'center')
           .css('border-bottom', '1px solid #e8e3e3')
           .css('padding', '5px 0');
    $menu.append($addText);
    $addText.on('click', () => {
      this._checkNeedBackStatus = true;
      let $prevBorders = this._findPrevBorders($border);
      let lastBorderIndex = $prevBorders.length - 1;
      let $baiscBorder = lastBorderIndex >= 0 ? $($prevBorders[lastBorderIndex]) : $border;
      let $currentParagraph = getParentByClassname($baiscBorder, 'js-paragraph-view');
      if ($currentParagraph) {
        let index = $currentParagraph.attr('data-index');
        let pageIndex = parseInt(index.split('-')[0]);
        if (pageIndex > -1) {
          let currentPage = this.pages[pageIndex];
          if (currentPage) {
            let $paragraph = $('<div class="js-paragraph-view"></div>');
            $paragraph.append(gennerOneLine());
            $paragraph.css('margin', 0)
                      .css('padding', '0 25px')
                      .css('outline', 'none')
                      .css('box-sizing', 'border-box')
                      .css('white-space', 'pre-wrap')
                      .css('line-height', '1.75')
                      .css('text-align', 'left');
            let index = pageIndex + '-' + currentPage.paragraphIndex++;
            $paragraph.attr('data-index', index);
            $paragraph.insertBefore($currentParagraph);
            setTimeout(() => {
              $paragraph.focus();
            }, 100)
          }
        } else {
          console.log('error');
        }
      } else {
        console.log('error')
      }

      $menu[0].blur();
    })
    // 添加文本框结束
    // 删除开始
    // let $delete = $('<div>删除框</div>');
    // $delete.css('cursor', 'pointer')
    //        .css('text-align', 'center')
    //        .css('padding', '5px 0');
    // $menu.append($delete);

    // $delete.on('click', () => {
    //   this._checkNeedBackStatus = true;
    //   // 如果是第一栏第一个框不允许删除
    //   let $currentParagraph = getParentByClassname($border, 'js-paragraph-view');
    //   let currentParagphChilds = $currentParagraph[0].childNodes;
    //   if (currentParagphChilds.length === 1 && $(currentParagphChilds[0]).equal($border)) {
    //     let $currentColumn = getParentByClassname($border, 'js-column');
    //     let $paragraphs = $currentColumn.find('.js-paragraph-view');
    //     let firstParagraph = this._cycleFindParagraph($paragraphs[0]);
    //     if (firstParagraph && $(firstParagraph).equal($currentParagraph)) {
    //       let pageIndex =  parseInt($currentColumn.attr('data-column-i'));
    //       if (this.pages.length === 1 && pageIndex === 0) {
    //         this.editor.message.showMessage('第一页第一个框不能删除！');
    //         $menu[0].blur();
    //         return false;
    //       }
    //     }
    //   }
    //   let $prevBorders = this._findPrevBorders($border);
    //   let $nextBorders = this._findNextBorders($border);
    //   let lastBorderIndex = $prevBorders.length - 1;
      
    //   $prevBorders.forEach(item => {
    //     let $item = $(item);
    //     let $paragraphItem = $item.parent();  
    //     $item.remove();
    //     if (isEmptyElement($paragraphItem)) {
    //       $paragraphItem.remove();
    //     }
    //   })

    //   $nextBorders.forEach(item => {
    //     let $item = $(item);
    //     let $paragraphItem = $item.parent();
    //     $item.remove();
    //     if (isEmptyElement($paragraphItem)) {
    //       $paragraphItem.remove();
    //     }
    //   })

    //   let $paragraphBorderItem = $border.parent();

    //   $border.remove();
    //   if (isEmptyElement($paragraphBorderItem)) {
    //     $paragraphBorderItem.remove();
    //   }
    //   $menu[0].blur();

    //   this._checkRemovePage();

    //   setTimeout(() => {
    //     this._checkoutContentChangeByFun();
    //   }, 100)
    // })

    // 删除结束
  },
 /**
 * function 获取前一栏
 */
  _getPrevColumn: function ($column) {
    let prev = $column[0].previousSibling;
    let pageIndex = parseInt($column.attr('data-column-page')) - 1;
    if (prev) {
      if (prev.className && prev.className.indexOf('js-column') > -1) {
        return $(prev);
      } else {
        if (pageIndex < 0) {
          return false;
        }

        let prevPage = this.pages[pageIndex];
        let lastColumnIndex = prevPage.$colums.length - 1;
        return prevPage.$colums[lastColumnIndex] || false;
      }
    } else {
      if (pageIndex < 0) {
        return false;
      }

      let prevPage = this.pages[pageIndex];
      let lastColumnIndex = prevPage.$colums.length - 1;
      return prevPage.$colums[lastColumnIndex] || false;
    }
  },
  // 获得该框以上的框
  _findPrevBorders: function ($border) {
    let $paragraph = getParentByClassname($border, 'js-paragraph-view');
    let index = $paragraph.attr('data-index');
    // 如果上一段在同一栏，则返回空
    let prevParagraph = $paragraph[0].previousSibling;
    if (prevParagraph && prevParagraph.className && prevParagraph.className.indexOf('js-paragraph-view') > -1) {
      let firstContent = prevParagraph.firstChild;
      if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') < 0) {
        return [];
      }
    }


    // 获取上一栏的最后一段
    let $lastParagraph = this._getPrevColumnLastParagraph($paragraph);

    if (!$lastParagraph) {
      return [];
    }

    let prevIndex = $lastParagraph.attr('data-index');
    let prevLastParagraph = $lastParagraph.attr('class');


    if (index === prevIndex) {
      let borders = [];
      let lastBorder = $lastParagraph[0].lastChild;
      borders.push(lastBorder);
      if ($lastParagraph[0].childNodes.length === 1 && lastBorder.className && lastBorder.className.indexOf('js-lsiten-border') > -1 && prevLastParagraph.indexOf('js-split-paragraph') > -1) {
        let prevBorders = this._findPrevBorders($(lastBorder));
        borders = borders.concat(prevBorders);
      }
      return borders;
    } else {
      return [];
    }

  },
  // 获取该框以上的问题
  _findPrevQustions: function ($border, type) {
    if ( type === 4) {
      let $title = $border.find('.lsiten-title');
      if ($title.length) {
        return [];
      }
    }

    let $paragraph = getParentByClassname($border, 'js-paragraph-view');
    let index = $paragraph.attr('data-index');
    // 如果上一段在同一栏，则返回空
    let prevParagraph = $paragraph[0].previousSibling;
    if (prevParagraph && prevParagraph.className && prevParagraph.className.indexOf('js-paragraph-view') > -1) {
      let firstContent = prevParagraph.firstChild;
      if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') < 0) {
        return [];
      }
    }


    // 获取上一栏的最后一段
    let $lastParagraph = this._getPrevColumnLastParagraph($paragraph);

    if (!$lastParagraph) {
      return [];
    }

    let prevIndex = $lastParagraph.attr('data-index');
    let prevLastParagraph = $lastParagraph.attr('class');

    if (index === prevIndex) {
      let lastBorder = $lastParagraph[0].lastChild;
      let borderContent = lastBorder.firstChild;
      if (borderContent) {
        let lastQuestion = borderContent.lastChild;
        let questionType = parseInt(lastQuestion.getAttribute('data-type'));
        let questions = [];
        if (type === questionType) {
          questions.push(lastQuestion);
  
          if (borderContent.childNodes.length === 1 && prevLastParagraph.indexOf('js-split-paragraph') > -1) {
            let prevQuestions = this._findPrevQustions($(lastBorder), type);
            questions = questions.concat(prevQuestions);
          }
          return questions;
        } else {
          return [];
        }
      } else {
        return [];
      }

    } else {
      return [];
    }
    
  },
  // 获取上一栏的最后一段
  _getPrevColumnLastParagraph: function ($paragraph) {
      let $colum = getParentByClassname($paragraph, 'js-column');
      if (!$colum) {
        return null;
      }
  
      let $prevColumn = this._getPrevColumn($colum);
  
      if (!$prevColumn) {
        return null;
      }
  
      let $paragraphs = $prevColumn.find('.js-paragraph-view');
  
      if (!$paragraphs.length) {
        return null;
      }
  
      let lastIndex = $paragraphs.length - 1;
  
      let lastParagraph = $paragraphs[lastIndex];
  
      let first = lastParagraph.firstChild;
      if (first.className && first.className.indexOf('js-answer-header') > -1) {
        return null;
      }
  
      return $(lastParagraph);
  
    },
  
  _findNextBorders: function ($border) {
      let $paragraph = getParentByClassname($border, 'js-paragraph-view');
      let index = $paragraph.attr('data-index');
      // 如果上一段在同一栏，则返回空
      let nextParagraph = $paragraph[0].nextSibling;
      if (nextParagraph && nextParagraph.className && nextParagraph.className.indexOf('js-paragraph-view') > -1) {
        let firstContent = nextParagraph.firstChild;
        if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') < 0) {
          return [];
        }
      }
  
      // 获取下一栏的第一段
      let $firstParagraph = this._getNextColumnfirstParagraph($paragraph);
  
      if (!$firstParagraph) {
        return [];
      }
  
      let nextIndex = $firstParagraph.attr('data-index');
  
      let nextFirstParagraph = $firstParagraph.attr('class');
  
      if (index === nextIndex) {
        let borders = [];
        let firstBorder = $firstParagraph[0].firstChild;
        borders.push(firstBorder);
        if ($firstParagraph[0].childNodes.length === 1 && firstBorder.className && firstBorder.className.indexOf('js-lsiten-border') > -1 && nextFirstParagraph.indexOf('js-split-paragraph') > -1) {
          let nextBorders = this._findNextBorders($(firstBorder));
          borders = borders.concat(nextBorders);
        }
        return borders;
      } else {
        return [];
      }
  },
  // 获取该框以下的问题
  _findNextQustions: function ($border, type) {
      let $paragraph = getParentByClassname($border, 'js-paragraph-view');
      let index = $paragraph.attr('data-index');
      // 如果上一段在同一栏，则返回空
      let nextParagraph = $paragraph[0].nextSibling;
      if (nextParagraph && nextParagraph.className && nextParagraph.className.indexOf('js-paragraph-view') > -1) {
        let firstContent = nextParagraph.firstChild;
        if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') < 0) {
          return [];
        }
      }
  
      // 获取下一栏的第一段
      let $firstParagraph = this._getNextColumnfirstParagraph($paragraph);
  
      if (!$firstParagraph) {
        return [];
      }
  
      let nextIndex = $firstParagraph.attr('data-index');
  
      let nextFirstParagraph = $firstParagraph.attr('class');
  
      if (index === nextIndex) {
        let firstBorder = $firstParagraph[0].firstChild;
        let borderContent = firstBorder.firstChild;
        if (borderContent) {
          let firstQuestion = borderContent.firstChild;
          let questionType = parseInt(firstQuestion.getAttribute('data-type'));
          let questions = [];
          if (type === questionType) {
            questions.push(firstQuestion);
    
            if (borderContent.childNodes.length === 1 && nextFirstParagraph.indexOf('js-split-paragraph') > -1) {
              let nextQuestions = this._findNextQustions($(firstBorder), type);
              questions = questions.concat(nextQuestions);
            }
            return questions;
          } else {
            return [];
          }
        } else {
          return [];
        }
  
      } else {
        return [];
      }
  },
  
  // 获取下一栏的第一段
  _getNextColumnfirstParagraph: function ($paragraph) {
      let $colum = getParentByClassname($paragraph, 'js-column');
      if (!$colum) {
        return null;
      }
  
      let $nextColumn = this._getNextColumn($colum);
  
      if (!$nextColumn) {
        return null;
      } 
  
      let $paragraphs = $nextColumn.find('.js-paragraph-view');
  
      if (!$paragraphs.length) {
        return null;
      }
  
      let firstParagraph = this._cycleFindParagraph($paragraphs[0]);
      if (!firstParagraph) {
        return null;
      }
  
      let firstContent = firstParagraph.firstChild;
      if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') > -1) {
        return null;
      }
  
      return $(firstParagraph);
  },
  
  
  // 循环查找段落 
  _cycleFindParagraph: function (paragraph) {
      if (!paragraph) {
        return null;
      }
      let i = 0;
      while (i < 100 && (!paragraph.className || paragraph.className.indexOf('js-paragraph-view') < 0)) {
        let temp = paragraph.nextSibling;
        paragraph.parentNode.removeChild(paragraph);
        paragraph = temp;
         i++;
      }
  
  
      let firstContent = paragraph.firstChild;
      if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') > -1) {
        let temp = paragraph.nextSibling;
        paragraph = temp; 
        return this._cycleFindParagraph(paragraph);
      }
       return paragraph || null;
  },
   /** 
   * function 内容变化事件监听
  */
  _contentchangeEvent: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('addContent', function (e) {
      _this._checkEveryPageOut();
    })

    Jquery($textElem[0]).on('removeContent', function (e) {
      _this._checkEveryPageBack();
    })
  },
  /** 
   * function del键监听
  */
  _delEvent: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('keydown', function (e) {       
      if (e.keyCode !== 46) {
        // 不是回车键
        return '';
      }
      if (getParentByClassname($(e.target), 'js-answer-header-title')) {
        return '';
      }
      let $currentLine = _this.editorPosition.currentLine;
      if ($currentLine) {
        let selection = _this.editor.selection;
        if (selection.isCartEnd($currentLine)) {
          let nextLine = $currentLine[0].nextSibling;
          if (nextLine && nextLine.className && nextLine.className.indexOf('js-lsiten-line') > -1 && isEmptyElement(nextLine)) {
            $(nextLine).remove();
            Jquery($textElem[0]).trigger('removeContent');
            return false;          
          } else {
            return false;
          }
        }
      } else {
        return false;
      }
    })
  },
  /** 
   * function 退格键监听
  */
  _backEvent () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('keydown', function (e) {       
      if (e.keyCode !== 8) {
        // 不是回车键
        return '';
      }
      if (getParentByClassname($(e.target), 'js-answer-header-title')) {
        return '';
      }
      let $currentLine = _this.editorPosition.currentLine;
      if ($currentLine) {
        if (isEmptyElement($currentLine[0])) {
          let prevLine = $currentLine[0].previousSibling;
          if (prevLine && prevLine.className && prevLine.className.indexOf('js-lsiten-line') > -1) {
            $currentLine.remove();
            prevLine.focus();
            Jquery($textElem[0]).trigger('removeContent');
            return false;            
          } else {
            return false;
          }
        }
      } else {
        return false;
      }
    })
  },
  /**
   * function 监听enter键事件
   */
  _enterEvent: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('keydown', function (e) {       
      if (e.keyCode !== 13) {
        // 不是回车键
        return '';
      }
      if (getParentByClassname($(e.target), 'js-answer-header-title')) {
        return '';
      }
      let $currentLine = _this.editorPosition.currentLine;
      if ($currentLine) {
        let $newLine = gennerOneLine();
        $newLine.insertAfter($currentLine);
        $newLine.focus();
      }

      Jquery($textElem[0]).trigger('addContent');
      return false;
    })
  },
  /**
   * function 实时保存当前编辑信息
   */
  _realtimeSavePosition: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    function realTimeSave (e) {
      _this._updatePositionInfo(e);
    }

    Jquery($textElem[0]).on('compositionend', function () {       
      _this.inputType = false;
    })
    Jquery($textElem[0]).on('keyup',  realTimeSave);
    Jquery($textElem[0]).on('keydown',  realTimeSave);
    Jquery($textElem[0]).on('mousedown',  e => {
      // mousedown 状态下，鼠标滑动到编辑区域外面，也需要保存选区
      Jquery($textElem[0]).on('mouseleave', '.js-column', realTimeSave);
    })
    Jquery($textElem[0]).on('mouseup', '.js-column', e => {
        realTimeSave(e);
        // 在编辑器区域之内完成点击，取消鼠标滑动到编辑区外面的事件
        Jquery($textElem[0]).off('mouseleave', realTimeSave);
    })
  },
  /**
   * 更新当前编辑信息
   */
  _updatePositionInfo: function (e, $ele) {
    let $target = $ele || $(e.target || e.toElement);
    if (!$target) {
      return false;
    }
    let $border = getParentByClassname($target, 'js-lsiten-border');
    let $line = getParentByClassname($target, 'js-lsiten-line');
    let $paragraph = getParentByClassname($target, 'js-paragraph-view');
    if (!$paragraph) {
      return false;
    }

    let $currentColumn = getParentByClassname($paragraph, 'js-column');
    if (!$currentColumn) {
      console.log($target, console.log("%c数据错误！", "color:red"));
      return false;
    }
    this.currentPage = this.pages[parseInt($currentColumn.attr('data-column-page'))];
    if (!this.currentPage) {
      console.log($target, $currentColumn, $paragraph, console.log("%c数据错误！", "color:red"));
      return false;
    }
    this.currentPage.currentColumn = parseInt($currentColumn.attr('data-column-i'));

    this.editorPosition = {
      currentParagraph: $paragraph,
      currentColumn: this.currentPage.$colums[this.currentPage.currentColumn],
      currentPage: this.currentPage,
      currentBorder: $border,
      currentLine: $line
    }

    let selection = this.editor.selection;
    if ($line) {
      if (!selection.getRange() ||  !selection.getSelectionContainerElem()) {
        selection.createRangeByElem($line, true);
      } else {
        selection.saveRange();
      }
    } else {
      selection.setEmptyRange();
    }
  },
  _pagesWidth: {
    'A4': '793px',
    'A3': '1587px'
  },
  _pagesHeight: {
    'A4': '1122px',
    'A3': '1122px'
  },
  _pagesBreakHeight: {
    'A4': '1122',
    'A3': '1122'
  }
}
export default yzPageManager
